/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */

package annotators.hunalign;

import com.cybozu.labs.langdetect.LangDetectException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import objects.Alignment_DocLevel;

/** This class rewrites the output of Hunalign in the OPUS-style format.
 * (For OPUS-style format see the end of the README.txt file.)
 *
 * @author Eva Mujdricza-Maydt
 * @lastChange 20120626
 */
public class ExtractHunalignOutput {

	/** Converts the Hunalign latter-format alignment files into an OPUS-format 
	 * alignment output.
	 * 
	 * @param alignmentPath path to the Hunalign-format alignment files
	 * @param inputExtension extension of the Hunalign-format alignment files
	 * @param outputExtension extension of the converted OPUS-format alignment output
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws LangDetectException 
	 */
	public static void writeHunalignAlignmentsInOpusFormat(
		String alignmentPath,
		String inputExtension, String outputExtension)
		throws FileNotFoundException, IOException, LangDetectException {
		
		ArrayList<String> alignmentFileNames = utils.Helper.extractFileNamesByEnding_woEnding(
			alignmentPath, inputExtension);
		
		for (String fileStem : utils.Helper.sortAL_str(alignmentFileNames)) {
			System.out.println("- " + fileStem);
			
			String currentAlignmentFileName = alignmentPath + fileStem + inputExtension;
			Alignment_DocLevel alignment = eval.EvaluateCorpus.readInAlignment_hunalignFormat(
					currentAlignmentFileName, fileStem, fileStem);
			System.out.println("\t" + alignment.getSentAlignments().size() + " alignments");
			
			//output
			String opusFormatOutput = alignmentPath + fileStem + outputExtension;
			
			corpora.ExtractCorpus_forEvaluation.writeAlignmentFile_opusFormat(
				opusFormatOutput, alignment);
		}
	}
	
}
