/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */

package annotators.treetagger;

/** Often used variables for the TreeTagger. These variables could be changed.
 *
 * @author Eva Mujdricza-Maydt
 * @lastChange 20120626
 */
public class Constants_TreeTagger {


	/** NOTE that the TreeTagger uses ISO-8859-1 coding, 
	 * see http://perso.ens-lyon.fr/adrien.barbaresi/blog/?p=52 */
	public static String encoding = "ISO-8859-1"; 
	
	/** extension of the input files for TreeTagger */
	public static String EXT_TREETAGGER_INPUT_XML = ".ttinput.xml";
	/** extension of the output files of TreeTagger */
	public static String EXT_TREETAGGER_OUTPUT_XML = ".ttoutput.xml";
	/** the first part of the bash script file name for the TreeTagger. This
	 * name will be completed by the language name. */
	public static String EXTPART_TREETAGGER_BASHSCRIPT = "treetagger_bashScript_";
	/** The command line options of TreeTagger we use. */
	public static final String OPTIONS_TREETAGGER = "-token -lemma -no-unknown -sgml";
	/** the root element in the input and output xml-files of TreeTagger. */
	public static final String ELEMENT_sentences = "sentences";
	/** element for one sentence in the input and output xml-files of TreeTagger. */
	public static final String ELEMENT_s = "s";
	/** attribute for the sentence number in the sentence-element in the input 
	 * and output xml-files of TreeTagger*/
	public static final String ATTRIBUTE_id = "id";
	
	/*
	 * The blog entry of http://perso.ens-lyon.fr/adrien.barbaresi/blog/?p=52:


	 * A fast bash pipe for TreeTagger October 18, 2010, 19:02
	 *
	 * I have been working with the part-of-speech tagger developed at the IMS
	 * Stuttgart TreeTagger since my master thesis. It performs very well on
	 * german texts as one could easily suppose, since it was one of its primary
	 * purposes. One major problem is that it’s poorly documented, so I would
	 * like to share the way that I found to pass things to TreeTagger through a
	 * pipe.
	 *
	 * The first thing is that TreeTagger doesn’t take Unicode strings, as it
	 * dates back to the nineties. So you have to convert whatever you pass to
	 * ISO-8859-1, which the iconv software with the translit option set does
	 * very well. It means here "find an equivalent if the character cannot be
	 * exactly translated".
	 *
	 * Then you have to define the options that you want to use. I put the most
	 * frequent ones in the example. Benefits
	 *
	 * The advantage of a pipe is that you can clean the text while passing it
	 * to the tagger. Here is one way of doing it, by using the text editor sed
	 * to : 1. remove the trailing white lines 2. replace everything that’s more
	 * than one space by one space and 3. replacing spaces by new lines.
	 *
	 * This way the TreeTagger gets one word every new line, as required, which
	 * is very convenient I think. Starting from a text file, you get a word,
	 * its tag and a new line. Here is the code that I use :


#!/bin/bash
INPUT=~/file.txt
TAGGER=~/something/TreeTagger/bin/tree-tagger
OPTIONS="-token -lemma"
PARMFILE=~/something/TreeTagger/lib/german.par


< $INPUT sed -e '/^$/d' -e 's/\s+/\s/g' -e 's/ /\n/g' |
iconv --from-code=UTF-8 --to-code=ISO-8859-1//TRANSLIT |
$TAGGER $OPTIONS $PARMFILE |
iconv --from-code=ISO-8859-1 --to-code=UTF-8//TRANSLIT | ... >


	 * That’s all ! Please let me know if these lines proved useful.
	 *
	 * Update: You can also choose to use directly the UTF8-encoded model for
	 * German, which was apparently trained on the same texts and which should
	 * behave the same way (although I have no evidence for that).
	 */
}
