/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * after count every token pairs appearance times from all the training file
 * pairs, there are a two dimention arrays diceMatrix, which saves source and
 * target often words appearance times and an arrayList, which saves token pair
 * appearances for the cases either source or target is(are) commen words
 *
 * writeDiceFiles write these two data structure into file, so that the
 * diceMatrix and array can be rebuilded later.
 *
 * @author Huiqin Körkel-Qu
 */
public class writeDiceFiles {

	@SuppressWarnings("unchecked")
	public static void writeArray(ArrayList<HashMap<Integer, Integer>> sourceToTarget, File arrayFile) throws IOException {
		//System.out.println("now write array with size "+sourceToTarget.size());

		BufferedWriter SToTBW = new BufferedWriter(new FileWriter(arrayFile));

		for (int i = 0; i < sourceToTarget.size(); i++) {
			int sourceWord = i;
			HashMap<Integer, Integer> targetHash = sourceToTarget.get(i);
			if (null == targetHash || targetHash.size() == 0) {
				SToTBW.write(i + "\n");
				continue;
			}

			StringBuilder newLine = new StringBuilder();
			newLine.append(sourceWord + "");
			Iterator iter = targetHash.entrySet().iterator();
			while (iter.hasNext()) {
				Map.Entry m = (Map.Entry) iter.next();
				int word = (Integer) m.getKey();
				int times = (Integer) m.getValue();
				newLine.append(" " + word + " " + times + " ");
			}
			SToTBW.write(newLine + "\n");
		}
		SToTBW.close();
	}

	public static void writeMatrix(int[][] diceMatrix, File matrixFile) throws IOException {
		BufferedWriter matrixBW = new BufferedWriter(new FileWriter(matrixFile));

		for (int i = 0; i < diceMatrix.length; i++) {
			for (int j = 0; j < diceMatrix[i].length; j++) {
				matrixBW.write(diceMatrix[i][j] + " ");
			}

			matrixBW.write("\n");
			if (i % 100 == 0) {
				matrixBW.flush();
			}
		}

		matrixBW.flush();
		matrixBW.close();
	}

	public static boolean createDir(String Path) {
		File dir = new File(Path);

		if (!dir.isDirectory()) {
			boolean success = false;;
			success = dir.mkdirs();
			if (!success) {
				System.out.println("unsuccessful in creating dir " + Path);
				return false;
			}
		}
		return true;
	}
}