/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package general.orderAlign;

import java.io.IOException;
import java.util.ArrayList;

/**
 * given a arrayList of alignment, find the piece where source(or target)
 * indexes are not continus, but target(source) indexes are continuous, and add
 * 0:1 or 1:0 alignments to fill up .
 *
 * @author Huiqin Körkel-Qu
 */
public class addEmpty {

	/**
	 *
	 * @param allAlign: the given alignments
	 * @param maxS: max source index, for the case at end of alignments where
	 * are empty alignment.
	 * @param maxT: max target index save as for maxT
	 * @param maxRange: the "limitation of continues", if it is sent to be 1,
	 * must be exactly continuous. if it is sent to be m, 2 difference in
	 * indexes is allowed.
	 * @throws IOException
	 */
	public static void add(ArrayList<String> allAlign, int maxS, int maxT, int maxRange) throws IOException {
		int nullAlignNum = 0;
		String firstAlign = allAlign.get(0);
		ArrayList<String> newAlign = new ArrayList<String>();
		for (int i = 0; i < allAlign.size(); i++) {
			newAlign.add(allAlign.get(i));
		}
		int firstAlignS = Integer.parseInt(firstAlign.split(";")[0].split(" ")[0]);
		int firstAlignT = Integer.parseInt(firstAlign.split(";")[1].split(" ")[0]);

		//at the begining of the alignment, there is 0:1 or 1:0 alignment.
		//eg. 2:8 is the first line
		if (firstAlignS <= maxRange && firstAlignT >= Math.max(5, 2 * (maxRange - 1))) {
			for (int i = 0; i < firstAlignT - 1; i++) {
				newAlign.add(i, ";" + (i + 1));
				nullAlignNum += 1;
			}
		} else if (firstAlignS == 1 && firstAlignT != 1) {
			for (int i = 0; i < firstAlignT - 1; i++) {
				newAlign.add(i, ";" + (i + 1));
				nullAlignNum += 1;
			}

		} else {
		}

		//at the begining, many S are not aligned
		if (firstAlignS >= Math.max(2 * (maxRange - 1), 5) && firstAlignT <= maxRange) {
			// System.out.println("add s side");
			for (int i = 0; i < firstAlignS - 1; i++) {
				newAlign.add(i, (i + 1) + ";");
				nullAlignNum += 1;
			}
		} else if (firstAlignT == 1 && firstAlignS != 1) {
			for (int i = 0; i < firstAlignS - 1; i++) {
				newAlign.add(i, (i + 1) + ";");
				nullAlignNum += 1;
			}

		} else {
		}

		String lastAlign = allAlign.get(allAlign.size() - 1);
		int lastSNum = lastAlign.split(";")[0].split(" ").length;
		int lastTNum = lastAlign.split(";")[1].split(" ").length;
		int lastAlignS = Integer.parseInt(lastAlign.split(";")[0].split(" ")[lastSNum - 1]);
		int lastAlignT = Integer.parseInt(lastAlign.split(";")[1].split(" ")[lastTNum - 1]);

		//at the end  of the alignment, many T is not aligned
		if (lastAlignS >= maxS - maxRange && lastAlignT + Math.max(5, 2 * (maxRange - 1)) < maxT) {
			for (int i = 0; i < maxT - lastAlignT; i++) {
				newAlign.add(";" + (lastAlignT + i + 1));
				nullAlignNum += 1;
			}
		} else if (lastAlignS == maxS && lastAlignT != maxT) {
			for (int i = 0; i < maxT - lastAlignT; i++) {
				newAlign.add(";" + (lastAlignT + i + 1));
				nullAlignNum += 1;
			}
		} else {
		}

		//at the end, ,many S is not aligned
		if (lastAlignS + Math.max(5, 2 * (maxRange - 1)) < maxS && lastAlignT >= maxT - maxRange) {
			for (int i = 0; i < maxS - lastAlignS; i++) {
				newAlign.add((lastAlignS + i + 1) + ";");
				nullAlignNum += 1;
			}
		} else if (lastAlignT == maxT && lastAlignS != maxS) {
			for (int i = 0; i < maxS - lastAlignS; i++) {
				newAlign.add((lastAlignS + i + 1) + ";");
				nullAlignNum += 1;
			}
		} else {
		}
		
		//   in middle, there is empty

		String alignBefore = allAlign.get(0);


		for (int i = 1; i < allAlign.size(); i++) {
			String alignAfter = allAlign.get(i);

			int SNumBefore = alignBefore.split(";")[0].split(" ").length;
			int TNumBefore = alignBefore.split(";")[1].split(" ").length;
			int alignSBefore = Integer.parseInt(alignBefore.split(";")[0].split(" ")[SNumBefore - 1]);
			int alignTBefore = Integer.parseInt(alignBefore.split(";")[1].split(" ")[TNumBefore - 1]);

			int alignSAfter = Integer.parseInt(alignAfter.split(";")[0].split(" ")[0]);
			int alignTAfter = Integer.parseInt(alignAfter.split(";")[1].split(" ")[0]);

			// s side is nearly continuous aligned. but T side there is a big empty area.
			if (alignSAfter - alignSBefore <= maxRange && (alignTAfter - alignTBefore) >= Math.max(2 * (maxRange - 1), 5)) {
				for (int j = 0; j < alignTAfter - alignTBefore - 1; j++) {
					newAlign.add(";" + (alignTBefore + j + 1));
					nullAlignNum += 1;
				}
			} else if (alignSAfter - alignSBefore == 1 && (alignTAfter - alignTBefore) > 1) {
				for (int j = 0; j < alignTAfter - alignTBefore - 1; j++) {
					newAlign.add(";" + (alignTBefore + j + 1));
					nullAlignNum += 1;
				}
			} else {
			}

			//t side is nearly continuous aligned, but S side there is a big empty 

			if (alignTAfter - alignTBefore <= maxRange && (alignSAfter - alignSBefore) >= Math.max(2 * (maxRange - 1), 5)) {
				for (int j = 0; j < alignSAfter - alignSBefore - 1; j++) {
					newAlign.add((alignSBefore + j + 1) + ";");
					nullAlignNum += 1;
				}
			} else if (alignTAfter - alignTBefore == 1 && (alignSAfter - alignSBefore) > 1) {
				for (int j = 0; j < alignSAfter - alignSBefore - 1; j++) {
					newAlign.add((alignSBefore + j + 1) + ";");
					nullAlignNum += 1;
				}

			} else {
			}

			alignBefore = alignAfter;

		}
		newAlign = orderedAlign.order(newAlign);
		allAlign.clear();
		for (int i = 0; i < newAlign.size(); i++) {
			allAlign.add(newAlign.get(i));
		}
	}
}
