/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTrain;

import java.io.IOException;
import diceMatrix.train.allFileDice;
import diceMatrix.train.rawFileToDiceMatrix;

/**
 * crfTrain.main contains 3 parameters argcs[0] is the path of training data. it
 * contains 3 subdirector, /de, /en and /align all the training source files are
 * under /de all the training target files are under /en all the given alignment
 * of source and target files are under /align for a file pair, only if source
 * file, target file, and alignment file are all there, it will be used in
 * training.
 *
 *
 * argcs[1] is the patternName, this patternName should fit to the requirement
 * of wapiti tools. if the patternName contains "/", it shows that it is an
 * absolute PatterName. otherwise, we assume the pattern file is under argcs[0]
 * with the given name patternName.
 *
 *
 * argcs[2] is the modelName similar to patternName, if it contains "/", it is a
 * absolutename, otherwise use the path from argcs[0] and the given modelName as
 * the full modelName
 *
 * @author Huiqin Körkel-Qu
 */
public class crfTrain {

	String path;
	String patternName;
	String modelName;
	String alignAppendix;

	/**
	 *
	 * @param path
	 * @param patternName
	 * @param modelName
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public crfTrain(String path, String alignAppendex, String patternName, String modelName) throws IOException, InterruptedException {
		if (path.charAt(path.length() - 1) != '/') {
			path = path + "/";
		}
		this.path = path;
		if (patternName.contains("/")) {
			this.patternName = patternName;

		} else {
			this.patternName = this.path + patternName;
		}

		this.alignAppendix = alignAppendex;
		diceMatrix.myConst.extensionForAlign = alignAppendex;


		if (modelName.contains("/")) {
			this.modelName = modelName;

		} else {
			this.modelName = this.path + modelName;
		}
	}

	public static void main(String[] argcs) throws IOException, InterruptedException {
		if (argcs.length < 3) {
			System.out.println("the parameters are not enough!");
			System.exit(1);
		}
		crfTrain crfTrainer;
		//if the user input only 2 parameter, the model will be saved as path+"model";
		if (argcs.length == 3) {
			crfTrainer = new crfTrain(argcs[0], argcs[1], argcs[2], "model");
		} // if the using input 3 or more parameter, the first is the path,
		// the second is the pattern, and the last is the model. 
		else {
			crfTrainer = new crfTrain(argcs[0], argcs[1], argcs[2], argcs[3]);
		}

		String path[] = new String[1];
		path[0] = crfTrainer.path;
		System.out.println("get the dice matrix");
		//get the word pairs' appearance times, and word appear times.
		rawFileToDiceMatrix.main(path);
		System.out.println("get the dice values for each file pair");
		// calculate dice feature for each training file pair.
		allFileDice.main(path);
		System.out.println("get feature files for the training data");
		// get other features and the feature files for the training file pairs
		generalTraining.main(path);
		//crf training and get the model from crf training.
		System.out.println("train the crf model");
		String trainingCommand = "wapiti train -p " + crfTrainer.patternName + " " + crfTrainer.path + "training.txt" + " " + crfTrainer.modelName;
//		  System.out.println("training command is:"+trainingCommand);
		Process crfTrain = Runtime.getRuntime().exec(trainingCommand);
		crfTrain.waitFor();
		System.out.println("training end"); //E
	}
}
