/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTrain;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * dice feature is only create for the max 15 Fs after true,
 *
 * @author Huiqin Körkel-Qu
 */
public class filePair {

	String dicePath = "/";
	String sourcePath;
	String targetPath;
//	String alignPath;
	String featurePath;
	String featureFileName;
	File sFile, tFile;
//	File alignFile;
	File diceFile;
	Hashtable<String, ArrayList<String>> posTable;
//	HashMap<String,String> alignMap;
	int maxIdS = 1;
	int maxIdT = 1;

	// for training, create feature for alignment in diagonal.
	public filePair(String FileName, String path,
		Hashtable<String, ArrayList<String>> posTable) throws IOException {
		sourcePath = path + "de/";
		targetPath = path + "en/";
//		alignPath=path+"align/";
		featurePath = path + "feature/";
		dicePath = path + "dice/";

		this.sFile = new File(sourcePath + FileName + diceMatrix.myConst.extensionForFeature);
		this.tFile = new File(targetPath + FileName + diceMatrix.myConst.extensionForFeature);

//		this.alignFile=new File(alignPath+FileName+".align");
		this.diceFile = new File(dicePath + FileName + ".dice");
		this.posTable = posTable;
//		this.alignMap=this.createAlignMap();
		this.featureFileName = featurePath + FileName + ".feature";
	}

	//for one filePair	
	public boolean createFeature() {
//	 long start=System.currentTimeMillis();
		if (this.diceFile.isDirectory() || !diceFile.exists()) {
//		 System.out.println("Dice file "+ this.diceFile.getAbsolutePath()+" doesnot exist");
			return false;
		}
		try {

//		 long fileToArrayStart=System.currentTimeMillis();
			fileToArray sFileXml = new fileToArray(sFile.getAbsolutePath());
			fileToArray tFileXml = new fileToArray(tFile.getAbsolutePath());
			sentenceBean[] allSBeans = sFileXml.allBeans;
			sentenceBean[] allTBeans = tFileXml.allBeans;
			if (null == allSBeans || null == allTBeans) {
				return false;
			}

			maxIdS = sFileXml.maxId;
			maxIdT = tFileXml.maxId;
			if (maxIdS != allSBeans.length || maxIdT != allTBeans.length) {
				return false;
			}
//	 	long fileToArrayEnd=System.currentTimeMillis();
//        System.out.println("read two xml file costed "+(fileToArrayEnd-fileToArrayStart) +" seconds");

			FileWriter featureWriter;
//		System.out.println("featureFileName is "+featureFileName);
			File featureFile = new File(featureFileName);

			featureWriter = new FileWriter(featureFile);
			BufferedWriter bufferedFW = new BufferedWriter(featureWriter);

			sentenceBean sBean, tBean;
			sentenceBean sBeanNeibour1, sBeanNeibour2;
			sentenceBean tBeanNeibour1, tBeanNeibour2;
			FileReader fr = new FileReader(diceFile);
			BufferedReader br = new BufferedReader(fr);
			ArrayList<String> allFeatures = new ArrayList<String>();
//		long featureStart=System.currentTimeMillis();
			String currentLine;
			while (null != (currentLine = br.readLine())) {
				if (currentLine.length() == 0) {
					//System.out.println("find an empty line!");
					allFeatures.add("\n");
					continue;
				}
				if (currentLine.contains("#")) {
					continue;
				}
//		    System.out.println("current line is \n"+ currentLine);
				int sId = Integer.parseInt(currentLine.split("\\s+")[1].split(";")[0]);
				int tId = Integer.parseInt(currentLine.split("\\s+")[1].split(";")[1]);
				String diceFeature = currentLine.split("\\s+")[0];

				String diceIndex = currentLine.split("\\s+")[1];
				String diceLabel = currentLine.split("\\s+")[2];

				//source sentence bean with neibours	
//			long sBeanStart=System.currentTimeMillis();
				sBean = sFileXml.getSentenceBean(sId);
				if (null == sBean) {
					continue;
				}
//			System.out.println("sentence Id is "+sId);
//			System.out.println("the bean is +");
//			for(int i=0;i<sBean.lemmaWordSeq.length;i++)
//			{
//				System.out.print(sBean.lemmaWordSeq[i]+" ");
//			}
//			System.out.println();
//			 long sBeanEnd=System.currentTimeMillis();
//	         System.out.println("getOne sentence Bean costed"+(sBeanEnd-sBeanStart) +" seconds");
				if (sId - 1 > 1) {
					sBeanNeibour1 = sFileXml.getSentenceBean(sId - 1);

				} else {
					sBeanNeibour1 = null;
				}

				if (sId - 2 > 1) {
					sBeanNeibour2 = sFileXml.getSentenceBean(sId - 2);
				} else {
					sBeanNeibour2 = null;
				}

				//target sentence bean with neibours
				tBean = tFileXml.getSentenceBean(tId);
				if (null == tBean) {
					continue;
				}
				if (tId - 1 > 1) {
					tBeanNeibour1 = tFileXml.getSentenceBean(tId - 1);

				} else {
					tBeanNeibour1 = null;
				}

				if (tId - 2 > 1) {
					tBeanNeibour2 = tFileXml.getSentenceBean(tId - 2);
				} else {
					tBeanNeibour2 = null;
				}

				String[] feature1to1;
				String[] feature2to1;
				String[] feature3to1;
				String[] feature1to2;
				String[] feature1to3;

//			long mergeBeanStart=System.currentTimeMillis();
				sentenceBean sBean2 = sBean.addBean(sBeanNeibour1);
				sentenceBean sBean3 = sBean2.addBean(sBeanNeibour2);
//			 long mergeBeanEnd=System.currentTimeMillis();
//	         System.out.println("merge 3 sentence"+(mergeBeanEnd-mergeBeanStart) +" seconds");

				sentenceBean tBean2 = tBean.addBean(tBeanNeibour1);
				sentenceBean tBean3 = tBean2.addBean(tBeanNeibour2);

				beanPair BeanPair = new beanPair();

//			long featureStart=System.currentTimeMillis();
				feature1to1 = BeanPair.pairFeature(sBean, tBean, posTable);
//			 long featureEnd=System.currentTimeMillis();
//	         System.out.println("feature for one sen pair costed "+(featureEnd-featureStart) +" seconds");

				feature1to1 = BeanPair.pairFeature(sBean, tBean, posTable);
				feature1to2 = BeanPair.pairFeature(sBean2, tBean, posTable);
				feature1to3 = BeanPair.pairFeature(sBean3, tBean, posTable);
				feature2to1 = BeanPair.pairFeature(sBean, tBean2, posTable);
				feature3to1 = BeanPair.pairFeature(sBean, tBean3, posTable);
//			 long allfeatureEnd=System.currentTimeMillis();
//	         System.out.println("feature for one sen pair with neibours costed "+(allfeatureEnd-featureStart) +" seconds");
				String positionFeature = this.getPositionRatio(sId, tId);
//			long writeStart=System.currentTimeMillis();

				StringBuilder currentFeatures = new StringBuilder();
				currentFeatures.append(feature1to1[0] + " ").
					append(feature1to1[1] + " ").
					append(feature1to1[2] + " ").
					append(feature1to1[3] + " ").
					append(feature1to1[4] + " ").
					append(positionFeature + " ").
					append(diceFeature + " ");

//			bufferedFW.write(feature1to1[0]+" ");
//			bufferedFW.write(feature1to1[1]+" ");
//			bufferedFW.write(feature1to1[2]+" ");
//			bufferedFW.write(feature1to1[3]+" ");
//			bufferedFW.write(feature1to1[4]+" ");
//			bufferedFW.write(positionFeature+" ");
//			bufferedFW.write(diceBinFeature+" ");
				for (int i = 0; i < 5; i++) {
					currentFeatures.append(feature2to1[i]).append(" ");
				}

				for (int i = 0; i < 5; i++) {
					currentFeatures.append(feature3to1[i]).append(" ");
				}
				for (int i = 0; i < 5; i++) {
					currentFeatures.append(feature1to2[i]).append(" ");
				}

				for (int i = 0; i < 5; i++) {
					if (i != 4) {
						currentFeatures.append(feature1to3[i] + " ");
					} else {
						currentFeatures.append(feature1to3[4]);
					}
				}

				currentFeatures.append("\t");
				currentFeatures.append(diceIndex + "");
				currentFeatures.append("\t");
				currentFeatures.append(diceLabel);

				allFeatures.add(currentFeatures.toString());
			}
//		long featureEnd=System.currentTimeMillis();
//		System.out.println("get the feature values costed "+(featureEnd-featureStart)+" mini seconds");
			for (String features : allFeatures) {
				bufferedFW.write(features + "\n");
			}
//		bufferedFW.write(currentFeatures.toString());	        	  
			bufferedFW.write("\n");
			bufferedFW.flush();
			bufferedFW.close();
			featureWriter.close();
//		 long end=System.currentTimeMillis();
//		 System.out.println("write to file costed "+(end-featureEnd)+"mini seconds ");
//         System.out.println("create feature costed "+(end-start) +"mill seconds");

			return true;

		} catch (Exception o) {
			o.printStackTrace();
		}

		System.out.println("Exception error!");

		return false;
	}

	public String getPositionRatio(int sId, int tId) {
		double positionRatio = Math.abs(sId * 1.0 / maxIdS - tId * 1.0 / maxIdT);

		String binaryPosition = String.format("%1.1e", positionRatio);
		return binaryPosition;
	}

	public String toBinaryFeature(double inValue, int bitNum, String type) {
		//inValue is the value to be round, bitNum is how many bit will be keep after .

		if (type.equals("normal")) // for a value larger than 0.09999999999, use  intpart+ "-" + decimalpart
		{
			int expand = 1;
			for (int i = 0; i < bitNum; i++) {
				expand *= 10;
			}
			double expandValue = inValue * expand;
			//from http://topic.csdn.net/t/20041201/09/3604201.html
			java.text.NumberFormat formate = java.text.NumberFormat.getNumberInstance();
			formate.setMaximumFractionDigits(0);
			String output = formate.format(expandValue);

			return output.split("\\.")[0];
		} // for the value which is too large or too small, use scientific value. 
		else if (type.equals("scienceNum")) {

			String binaryFeature = String.format("%1.1e", inValue);

			return binaryFeature;
		} //this should not happen!
		else {
			return "0";
		}
	}
}
