/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package main;

/** The most important constants and common variables. 
 * Change them if necessary.
 *
 * @author Eva Mujdricza-Maydt
 */
public class Constants_Main {
	
	/**
	 * The enumeration of predefined processes you can start in the Main-class.
	 */
	public enum PROCESS {
		preproc,
		crftrain, 
		crftest,
		eval,
		signiftest,
		convert,
		html,
		normalizeopus
	}
	
	/** the character slash */
	public static final String SLASH = "/";
	/** the character backslash */
	public static final String BACKSLASH = "\\";

	/** language code for german texts */
	public static final String LANG_DE = "de";
	/** language code for english texts */
	public static final String LANG_EN = "en";
	
	/** language code for source texts */
	public static final String LANG_SRC = LANG_DE;
	/** language code for target texts */
	public static final String LANG_TRG = LANG_EN;

	/** folder name for source texts specified by the language code */
	public static final String FOLDER_SRC = LANG_SRC + SLASH;
	/** folder name for target texts specified by the language code */
	public static final String FOLDER_TRG = LANG_TRG + SLASH;

	/** "align" */
	public static final String ALIGNMENTS = "align";
	/** folder name for the alignment files */
	public static final String FOLDER_ALIGNMENTS = ALIGNMENTS + SLASH;
	/** folder name for the external sources we use */
	public static final String FOLDER_LIB = "lib" + SLASH;
	
	/** "tokens" */
	public static String TOKENS = "tokens";
	/** "lemmata" */
	public static String LEMMATA = "lemmata";
	
	/** The minimal number of the first sentence in a text, set to 1. 
	 * There should not be any sentence number lower than this number. */
	public static final int MINIMUMSENTNUMBER = 1;
	
	/** For OPUS corpus we build the file names from the original 
	 * source and target file names, 
	 * so that we keeps both of them and take a separator character between them:
	 * {@literal <sourcename><separator><targetname><fileextension>} */
	public static final String SEPARATOR_SRCTRG_FILENAME = "-";

	/** file extension for xml-files */
	public static final String EXT_XML = ".xml";
	/** file extension for plain text-files */
	public static final String EXT_TXT = ".txt";
	/** file extension for html-files */
	public static final String EXT_HTML = ".html";
	/** file extension for bash scripts */
	public static final String EXT_SH = ".sh";
	/** file extension for files compressed with gz */
	public static final String EXT_GZ = ".gz";
	/** file extension for xml-files compressed with gz */
	public static final String EXT_XMLGZ = ".xml.gz";
	/** file extension for text-files entailing the tokenized text */
	public static final String EXT_TEXT_TOKENS = ".text_tokens";
	/** file extension for text-files entailing the lemmata of the tokenized text */
	public static final String EXT_TEXT_LEMMATA = ".text_lemmata";
	
	/** file extension for system alignment files in opus format */
	public static String EXT_OTHEROPUSFORMATALIGNMENT = "";
	/** a default file extension for opus alignment files */
	public static final String EXT_OPUSALIGNMENT = ".opusAlign";
	/** a default file extension for gargantua alignment files */
	public static final String EXT_GARGANTUAALIGNMENT = ".gargantuaAlign";
	/** a default file extension for hunalign alignment files */
	public static final String EXT_HUNALIGNALIGNMENT = ".hunalignAlign";
	/** a default file extension for realigned hunalign alignment files */
	public static final String EXT_HUNALIGNALIGNMENT_REALIGNED = ".hunalignRealign";
	/** a default file extension for manual created golden alignment files */
	public static final String EXT_GOLDENALIGNMENT = ".goldAlign";
	/** a default file extension for system alignment files in opus format */
	public static final String EXT_SYSTEMALIGNMENT = ".sysAlign";
	/** a default file extension for champollion alignment files */
	public static final String EXT_CHAMPOLLIONALIGNMENT = ".champollionAlign";
	
}
