/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package trainDev;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;

/**
 * devide the feature file into training file and development file. 19/20 as
 * training data, 1/20 as development data
 *
 * @author Huiqin Körkel-Qu
 */
public class getTrainDevFiles {

	public static boolean get(String fileName) throws IOException {
		File file = new File(fileName);
		if (!file.isFile()) {
			System.out.println("the file to get the training and development file doesn't exist!");
			return false;
		}

		int lineNum = getTotalLines(fileName);

		int trainingNum = (int) Math.round(lineNum * (19.0 / 20));

		String trainingName = fileName + ".training";
		String devName = fileName + ".dev";
		FileWriter tW = new FileWriter(trainingName);
		FileWriter dW = new FileWriter(devName);
		BufferedWriter tBW = new BufferedWriter(tW);
		BufferedWriter dBW = new BufferedWriter(dW);
		FileReader inR = new FileReader(fileName);
		BufferedReader inBR = new BufferedReader(inR);

		int lineCount = 0;

		String currentLine;
		while (null != (currentLine = inBR.readLine())) {
			lineCount += 1;
			if (lineCount < trainingNum) {
				tBW.write(currentLine);
				tBW.write("\n");
			} else {
				dBW.write(currentLine);
				dBW.write("\n");
			}
		}
		tBW.close();
		dBW.close();
		inBR.close();
		return true;
	}

	/*
	 * from
	 * http://hi.baidu.com/wy494032874/blog/item/d4a5a0d457dc9416a08bb7cb.html
	 */
	private static int getTotalLines(String fileName) throws IOException {
		FileReader in = new FileReader(fileName);
		LineNumberReader reader = new LineNumberReader(in);
		String strLine = reader.readLine();
		int totalLines = 0;
		while (strLine != null) {
			totalLines++;
			strLine = reader.readLine();
		}
		reader.close();
		in.close();
		return totalLines;
	}

	public static void main(String[] argc) throws IOException {
		String fileName = argc[0];
		getTrainDevFiles.get(fileName);
	}
}
